from django.contrib.admin.views.main import ChangeList, ChangeListSearchForm
from django import forms


ADVANCED_SEARCH_FIELD = 'f'


class AdminSearchForm(ChangeListSearchForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Add advanced search field to the form
        self.fields.update({
            ADVANCED_SEARCH_FIELD: forms.CharField(required=False, strip=False)
        })


class CustomChangeList(ChangeList):
    search_form_class = AdminSearchForm

    def __init__(self, request, model, list_display, list_display_links,
                 list_filter, date_hierarchy, search_fields, list_select_related,
                 list_per_page, list_max_show_all, list_editable, model_admin, sortable_by):
        super().__init__(request, model, list_display, list_display_links,
                         list_filter, date_hierarchy, search_fields, list_select_related,
                         list_per_page, list_max_show_all, list_editable, model_admin, sortable_by)

        # Add the field data to be used in the template
        # To mark the list item as selected
        self.field_ = request.GET.get(ADVANCED_SEARCH_FIELD)

    def get_filters(self, request):
        """
        Override the get_filters method.
        Get all filter attributes from the parent get_filters method
        Remove the `f` (search field key) from the remaining lookup params
        Return filter attributes with the updated data

        :param request:
        :return:
        """

        # Get filters attributes from parent
        (
            filter_specs,
            has_filters,
            remaining_lookup_params,
            filters_use_distinct,
            has_active_filters,
        ) = super().get_filters(request)

        # If remaining lookup params has the advanced search field, remove the same from the dict
        # Otherwise it will IncorrectLookupParameters exception from
        # django.contrib.admin.views.main.py (Line 484)
        if remaining_lookup_params.get(ADVANCED_SEARCH_FIELD, None):
            del remaining_lookup_params[ADVANCED_SEARCH_FIELD]

        return (
            filter_specs,
            has_filters,
            remaining_lookup_params,
            filters_use_distinct,
            has_active_filters
        )



from django.contrib.admin.views.main import SEARCH_VAR
from django import template

register = template.Library()


@register.inclusion_tag('admin/custom_search_form.html', takes_context=False)
def advanced_search_form(cl):
    """
    Displays a search form for searching the list.
    """
    return {
        'cl': cl,
        'show_result_count': cl.result_count != cl.full_result_count,
        'search_var': SEARCH_VAR
    }



import operator
from functools import reduce

from django.contrib.admin import ModelAdmin
from django.contrib.admin.utils import lookup_needs_distinct
from django.core.exceptions import FieldDoesNotExist
from django.db import models
from django.db.models.constants import LOOKUP_SEP


class DjangoAdminSBF(ModelAdmin):
    """
    Extend this class to add advanced search in the admin panel
    """

    change_list_template = 'admin/custom_change_list.html'

    def get_changelist(self, request, **kwargs):
        from .admin_forms import CustomChangeList
        return CustomChangeList

    def get_search_results(self, request, queryset, search_term):

        # Apply keyword searches.
        def construct_search(field_name):
            if field_name.startswith('^'):
                return "%s__istartswith" % field_name[1:]
            elif field_name.startswith('='):
                return "%s__iexact" % field_name[1:]
            elif field_name.startswith('@'):
                return "%s__search" % field_name[1:]
            # Use field_name if it includes a lookup.
            opts = queryset.model._meta
            lookup_fields = field_name.split(LOOKUP_SEP)
            # Go through the fields, following all relations.
            prev_field = None
            for path_part in lookup_fields:
                if path_part == 'pk':
                    path_part = opts.pk.name
                try:
                    field = opts.get_field(path_part)
                except FieldDoesNotExist:
                    # Use valid query lookups.
                    if prev_field and prev_field.get_lookup(path_part):
                        return field_name
                else:
                    prev_field = field
                    if hasattr(field, 'get_path_info'):
                        # Update opts to follow the relation.
                        opts = field.get_path_info()[-1].to_opts
            # Otherwise, use the field with icontains.
            return "%s__icontains" % field_name

        query_field_params = request.GET.get('f', None)
        print('query_params_________: {}'.format(query_field_params))

        use_distinct = False
        search_fields = self.get_search_fields(request)
        print('search_fields________: {}'.format(search_fields))
        print('search term__________: {}'.format(search_term))
        if search_fields and search_term and query_field_params in search_fields:
            # orm_lookups = [construct_search(str(search_field))
            #                for search_field in search_fields]
            orm_lookups = [construct_search(str(query_field_params))]
            for bit in search_term.split():
                or_queries = [models.Q(**{orm_lookup: bit})
                              for orm_lookup in orm_lookups]
                queryset = queryset.filter(reduce(operator.or_, or_queries))
            use_distinct |= any(lookup_needs_distinct(self.opts, search_spec) for search_spec in orm_lookups)

        # traceback.print_stack()
        # return super().get_search_results(request, queryset, search_term)
        return queryset, use_distinct
